@@ -1,8 +1,8 @@
 /*
  *      gb.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: gb.c 1329 2007-02-24 11:41:56Z ntrel $
+ * $Id: gb.c 2788 2008-07-18 13:40:48Z eht16 $
  */
 
 /*
@@ -30,13 +30,10 @@
 #include <time.h>
 #include <signal.h>
 #include <unistd.h>
-#ifdef HAVE_FCNTL_H
-	#include <fcntl.h>
-#endif
 #include <gdk-pixbuf/gdk-pixdata.h>
 
 #define MAX_PICS 10
-#define LOOP_DELAY 200000	// micro seconds
+#define LOOP_DELAY 200000	/* micro seconds */
 
 #define IMAGE_LOGO			10
 #define IMAGE_BUTTON_UP		11
@@ -49,7 +46,6 @@ unsigned short iconset;
 GtkWidget *image1, *image2, *image3, *image4, *label1, *label2, *label3, *okbutton1, *textview1;
 gchar info_texts[4][50];
 gchar *help_text;
-gint random_fd;
 gboolean is_running;
 static GdkPixbuf **icons;
 
@@ -77,7 +73,7 @@ static void create_window(void)
 	gtk_window_set_position(GTK_WINDOW(gb_window), GTK_WIN_POS_CENTER);
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(gb_window), TRUE);
 	gtk_window_set_type_hint(GTK_WINDOW(gb_window), GDK_WINDOW_TYPE_HINT_DIALOG);
-	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(gb_window), TRUE);
+	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(gb_window), FALSE);
 	gtk_window_set_modal(GTK_WINDOW(gb_window), TRUE);
 	gtk_window_set_skip_pager_hint(GTK_WINDOW(gb_window), TRUE);
 	gtk_window_set_icon(GTK_WINDOW(gb_window), icons[IMAGE_LOGO]);
@@ -143,9 +139,9 @@ static void create_window(void)
 	gtk_box_pack_start(GTK_BOX(hbox2), button5, FALSE, FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(button5), 5);
 
-	g_signal_connect((gpointer) button1, "clicked", G_CALLBACK(on_button1_clicked), NULL);
-	g_signal_connect((gpointer) button4, "clicked", G_CALLBACK(on_button4_clicked), NULL);
-	g_signal_connect((gpointer) button5, "clicked", G_CALLBACK(on_button5_clicked), NULL);
+	g_signal_connect(button1, "clicked", G_CALLBACK(on_button1_clicked), NULL);
+	g_signal_connect(button4, "clicked", G_CALLBACK(on_button4_clicked), NULL);
+	g_signal_connect(button5, "clicked", G_CALLBACK(on_button5_clicked), NULL);
 
 	gtk_widget_grab_focus(button4);
 }
@@ -208,39 +204,6 @@ static gint destroydialog(GtkWidget *widget, gpointer data)
 }
 
 
-static void initialise_random_numbers(void)
-{
-#if defined(HAVE_FCNTL_H) && (defined(HAVE_DEVURANDOM) || defined(HAVE_DEVRANDOM))
-# ifdef HAVE_DEVURANDOM
-	random_fd = open("/dev/urandom", O_NONBLOCK | O_RDONLY);
-# elif HAVE_DEVRANDOM
-	random_fd = open("/dev/random", O_NONBLOCK | O_RDONLY);
-# endif
-#else
-    srand(time(NULL) * getpid());
-#endif
-}
-
-
-static gint random_number(gint max)
-{
-#if defined(HAVE_FCNTL_H) && (defined(HAVE_DEVURANDOM) || defined(HAVE_DEVRANDOM))
-    if (max <= 255)
-	{
-		unsigned char byte;
-		while (read(random_fd, &byte, 1) == 1)
-		{
-	    	if (byte < max)
-				return (byte % max);
-		}
-    }
-    return 0;
-#else
-	return (((unsigned short) rand()) % max);
-#endif
-}
-
-
 static void on_button5_clicked(GtkButton *button, gpointer user_data)
 {
     gb_destroyapp(GTK_WIDGET(button), user_data);
@@ -271,30 +234,29 @@ static void on_button1_clicked(GtkButton *button, gpointer user_data)
 {
     unsigned short erg_a, erg_b, erg_c, i, l, m, n, loops;
 
-	if (is_running) return;	// prevent multiple clicks
+	if (is_running) return;	/* prevent multiple clicks */
 	is_running = TRUE;
-	// change button-image
+	/* change button-image */
 	gtk_image_set_from_pixbuf(GTK_IMAGE(image4), icons[IMAGE_BUTTON_DOWN]);
 
 	lap++;
 	update_labels(gb_window, FALSE, 3);
 
-	l = random_number(MAX_PICS);
-	m = random_number(MAX_PICS);
-	n = random_number(MAX_PICS);
-	erg_a = random_number(MAX_PICS);
-	erg_b = random_number(MAX_PICS);
-	erg_c = random_number(MAX_PICS);
-	//printf("Ani: %d, %d, %d\tErg: %d, %d, %d\n\n", l, m, n, erg_a, erg_b, erg_c);
+	l = g_random_int_range(0, MAX_PICS);
+	m = g_random_int_range(0, MAX_PICS);
+	n = g_random_int_range(0, MAX_PICS);
+	erg_a = g_random_int_range(0, MAX_PICS);
+	erg_b = g_random_int_range(0, MAX_PICS);
+	erg_c = g_random_int_range(0, MAX_PICS);
 
-	// assign icons
+	/* assign icons */
 	loops = 30;
 	for(i = 0; i < loops; i++)
 	{
 		if (l > 9) l = 0;
 		if (m > 9) m = 0;
 		if (n > 9) n = 0;
-		// simulate stopping of first and second slot
+		/* simulate stopping of first and second slot */
 		if (i < (loops - 10))
 			gtk_image_set_from_pixbuf(GTK_IMAGE(image1), icons[l]);
 		else
@@ -308,7 +270,7 @@ static void on_button1_clicked(GtkButton *button, gpointer user_data)
 		l++;
 		m++;
 		n++;
-		// refresh()-replacement
+		/* refresh()-replacement */
 		while (g_main_context_iteration(NULL, FALSE));
 		g_usleep(LOOP_DELAY);
 	}
@@ -319,7 +281,7 @@ static void on_button1_clicked(GtkButton *button, gpointer user_data)
 
 	update_labels(gb_window, FALSE, i);
 
-	// change button image
+	/* change button image */
 	gtk_image_set_from_pixbuf(GTK_IMAGE(image4), icons[IMAGE_BUTTON_UP]);
 	is_running = FALSE;
 
@@ -331,8 +293,8 @@ static void on_button4_clicked(GtkButton *button, gpointer user_data)
 	GtkWidget *dialog = create_help_dialog();
 	GtkTextBuffer *buffer;
 
-	g_signal_connect(G_OBJECT(dialog), "delete_event", G_CALLBACK(destroydialog), G_OBJECT(dialog));
-	g_signal_connect(G_OBJECT(okbutton1), "clicked", G_CALLBACK(destroydialog), G_OBJECT(dialog));
+	g_signal_connect(dialog, "delete-event", G_CALLBACK(destroydialog), G_OBJECT(dialog));
+	g_signal_connect(okbutton1, "clicked", G_CALLBACK(destroydialog), G_OBJECT(dialog));
 
 	buffer = gtk_text_buffer_new(NULL);
 	gtk_text_buffer_set_text(buffer, help_text, strlen(help_text));
@@ -363,7 +325,7 @@ static void update_labels(GtkWidget *window, gint init, gint won)
 	}
 	gtk_label_set_text(GTK_LABEL(label3), info_texts[won]);
 
-	snprintf(pts, 50, "Points: %4d\tRound: %2d\n", points, lap);
+	g_snprintf(pts, 50, "Points: %4d\tRound: %2d\n", points, lap);
 	gtk_label_set_text(GTK_LABEL(label1), pts);
 
 }
@@ -1251,24 +1213,21 @@ static void init_images(void)
 {
 	gint i;
 
-	initialise_random_numbers();
-
-	// define start images
-	i = random_number(MAX_PICS) + 1;
+	/* define start images */
+	i = g_random_int_range(0, MAX_PICS);
 	gtk_image_set_from_pixbuf(GTK_IMAGE(image1), icons[i]);
-	i = random_number(MAX_PICS) + 1;
+	i = g_random_int_range(0, MAX_PICS);
 	gtk_image_set_from_pixbuf(GTK_IMAGE(image2), icons[i]);
-	i = random_number(MAX_PICS) + 1;
+	i = g_random_int_range(0, MAX_PICS);
 	gtk_image_set_from_pixbuf(GTK_IMAGE(image3), icons[i]);
 }
 
 
-// main exit function
+/* main exit function */
 gint gb_destroyapp(GtkWidget *widget, gpointer gdata)
 {
 	if (is_running) return TRUE;
 	if (GTK_IS_WINDOW(gb_window)) gtk_widget_destroy(gb_window);
-	if (random_fd != -1) close(random_fd);
 	gb_window = NULL;
 	return FALSE;
 }
@@ -1276,10 +1235,9 @@ gint gb_destroyapp(GtkWidget *widget, gpointer gdata)
 
 void gb_start_easteregg(void)
 {
-	random_fd = -1;
 	load_images();
 	create_window();
-	g_signal_connect(G_OBJECT(gb_window), "delete_event", G_CALLBACK(gb_destroyapp), NULL);
+	g_signal_connect(gb_window, "delete-event", G_CALLBACK(gb_destroyapp), NULL);
 
 	points = 0;
 	lap = 0;
