@@ -19,24 +19,31 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: navqueue.h 2150 2008-01-06 18:11:57Z eht16 $
+ * $Id: navqueue.h 2705 2008-06-18 17:03:08Z ntrel $
  */
 
+/**
+ *  @file navqueue.h
+ * Simple code navigation
+ **/
+
 
 #ifndef GEANY_NAVQUEUE_H
 #define GEANY_NAVQUEUE_H 1
 
 
-void navqueue_init();
+void navqueue_init(void);
 
-void navqueue_free();
+void navqueue_free(void);
+
+void navqueue_remove_file(const gchar *filename);
 
 
-gboolean navqueue_goto_line(gint new_idx, gint line);
+gboolean navqueue_goto_line(GeanyDocument *old_doc, GeanyDocument *new_doc, gint line);
 
-void navqueue_go_back();
+void navqueue_go_back(void);
 
-void navqueue_go_forward();
+void navqueue_go_forward(void);
 
 
 #endif
